Attribute VB_Name = "modCommonDialogs"
Option Explicit
DefInt A-Z

' ###########################################
'
'                      MODUL "CommonDialog"
'
'                     (c) Ingo Steinhaus 2000
'                     ingo.steinhaus@gmx.de
'
'  Funktionen zur Anzeige der Windows-Standarddialoge "ffnen"
'  und "Speichern"
'
'  Dieses Modul ist urheberrechtlich geschtzte Freeware.
'  Die originale Copyright-Meldung darf nicht entfernt oder ver-
'  ndert werden. Der Quelltext darf nicht verndert werden-
'
'############################################

Private Const MAX_PATH = 260

'**********************************************************
Rem Die Datenstruktur "OpenFilename" dient der Konfiguration
Rem des Dialogs.

Private Type OpenFilename
    lStructSize As Long
    'Gre der Datenstruktur. Kann mit Len() bestimmt werden.
    hWndOwner As Long
    'Handle des Besitzers (mit GetActiveWindow() abfragen).
    hInstance As Long
    'Handle der Dialogfeldvorlage, wenn OFN_ENABLETEMPLATEHANDLE
    'in Flags gesetzt ist. Wenn OFN_EXPLORER gesetzt ist, wird der
    'Dialog vom Standardialog des Explorers abgeleitet. Andernfalls
    'wird ein Dialog im Windows-3.x-Stil erzeugt.
    lpstrFilter As String
    'Ein VB-String mit paarweise angeordneten nullterminierten Strings.
    'Der letzte nullterminierte String mu mitr einem weiteren NULL-Zeichen
    'abgeschlossen werden.
    'Ein Filter besteht aus zwei nullterminierten Strings. Der erste enthlt
    'die Zeichenkette, die im Kombifeld "DateiTyp" angezeigt wird, der zweite
    'die zugehrtigen Dateimasken wie z. B. "*.doc".
    'Beispiel: "Word-Dokumente" + Chr(0) + "*.doc" + Chr(0)
    'Sie knnen mehrere Dateimasken durch Semikola abtrennen.
    'Beispiel: "Grafiken" + Chr(0) + "*.bmp;*.jpg;*.gif" + Chr(0)
    lpstrCustomFilter As String
    'Ein VB-String mit dem im Kombifeld "DateiTyp" ausgewhlten Filter.
    nMaxCustFilter As Long
    'Die Gre von lpstrCustomFilter.
    nFilterIndex As Long
    'Der 1-basierte Index des im Kombifeld "DateiTyp" ausgewhlten Filters.
    lpstrFile As String
    'Ein VB-String mit dem ausgewhlten Dateinamen inkl. Laufwerk und Pfad.
    'Der String mu vorher in der entsprechenden Gre erzeugt werden.
    'Er kann vor dem Aufruf des Dialogs mit dem Namen einer existierenden
    'Datei belegt werden.
    nMaxFile As Long
    'Die Gre von lpstrFile.
    lpstrFileTitle As String
    'Ein VB-String mit dem ausgewhlten Dateinamen ohne Laufwerk und Pfad.
    nMaxFileTitle As Long
    'Die Gre von lpstrFileTitle.
    lpstrInitialDir As String
    'Ein VB-String mit dem Pfadnamen des Ordners, dessen Inhalt der Dialog
    'beim Anzeigen darstellen soll.
    lpstrTitle As String
    'Ein VB-String mit Titel des Dialogfeldes.
    Flags As Long
    'Flags, die die Anzeigeoptionen des Dialogfeldes bestimmen
    nFileOffset As Integer
    'Index zum Beginn des ersten Dateinamens in lpstrFile.
    nFileExtension As Integer
    'Index zum Beginn der Dateierweiterung in lpstrFile.
    lpstrDefExt As String
    'Die Standarderweiterung, die an einen Dateinamen vergeben wird, wenn
    'er keine Erweiterung besitzt.
    lCustData As Long
    'Ein Zeiger auf anwendungsspezifiasche Daten, fr die Rckruffunktion.
    lpfnHook As Long
    'Adresse einer Rckruffunktion, die in der Anwendung definiert wird.
    'Sie knnen hier NULL eintragen.
    lpTemplateName As String
    'Der Name der Dialogfeldvorlage (siehe hInstance)
End Type

'**********************************************************
Rem Die folgenden Konstanten sind die erlaubten Werte fr
Rem OpenFilename->Flags.

Private Const OFN_ALLOWMULTISELECT = &H200
'Zeigt ein Dialogfeld mit der Mglichkeit, mehrere Dateien auszuwhlen.
'In diesem Fall enthlt lpstrFile den Pfad und anschlieend alle Dateinamen.
'nFileOffset zeigt auf den Index des ersten Dateinamens nach der Pfadangabe.
'lpstrFile enthlt alle Dateinamen durch Chr(0) getrennt. Em Ende folgt ein
'zweites Chr(0). Bei alten Win-3.x-Dialoge) sind die Dateinamen durch
'Leerzeichen getrennt. Diese Variante kennt keine langen Dateinamen.

Private Const OFN_CREATEPROMPT = &H2000
'Zeigt eine Meldung, wenn die Datei nicht existiert und fragt den Anwender, ob
'sie erzeugt werden soll.

Private Const OFN_ENABLEHOOK = &H20
'Aktiviert die Rckruffunktion lpfnHook.

Private Const OFN_ENABLETEMPLATE = &H40
'Aktiviert die Dialogfeldvorlage.

Private Const OFN_ENABLETEMPLATEHANDLE = &H80
'Aktiviert die Dialogfeldvorlage.

Private Const OFN_EXPLORER = &H80000
'Nutzt Explorer-Dialoge. Diese Einstellung ist die Vorgabe, selbst wenn Sie
'dieses Flag nicht angeben. Fr alte Win-3.x-Dialoge mssen Sie das Flag
'lschen.
'Sie mssen es in den folgenden Fllen setzen:
'- bei OFN_ALLOWMULTISELECT.
'- wenn Sie Dialogfeldvorlagen und Rckruffunktionen benutzen.

Private Const OFN_EXTENSIONDIFFERENT = &H400&
'Gibt an, dass der Anwender einen Dateinamen mit einer anderen Erweiterung
'als lpstrDefExt eingeben kann.

Private Const OFN_FILEMUSTEXIST = &H1000
'Gibt an, dass der Anwender nur die Namen von existierenden Dateien eingeben
'kann. Andernfalls wird eine Warnmeldung ausgegeben.
'OFN_PATHMUSTEXIST mu ebenfalls gesetzt werden.

Private Const OFN_HIDEREADONLY = &H4&
'Versteckt das Kontrollkstchen "Nur lesen".

Private Const OFN_LONGNAMES = &H200000
'Aktiviert die Untersttzung von langen Dateinamen in den alten Win-3.x-Dialogen.

Private Const OFN_NOCHANGEDIR = &H8&
'Stellt das ursprngliche Verzeichnis bei Ende des Dialoges wieder her, wenn
'der Anwender anderes Verzeichnis eingestellt hat.

Private Const OFN_NODEREFERENCELINKS = &H100000
'Weist das Dialogfeld an, bei einer markierten Verknpfung Namen und Pfad der
'Verknpungsdatei zurckzugeben, snstatt Namen und Pfad der Datei, auf die die
'Verknpfung verweist.

Private Const OFN_NOLONGNAMES = &H40000
'Deaktiviert die Untersttzung von langen Dateinamen in den alten
'Win-3.x-Dialogen.

Private Const OFN_NONETWORKBUTTON = &H20000
'Versteckt die Schaltflche "Netzwerk".

Private Const OFN_NOTESTFILECREATE = &H10000
'Gibt an, dass keine Testdatei erzeugt wird, bevor der Dialog endet. In diesem
'Fall berprft das Dialogfeld nicht auf Schreibschutz, Platzmangel auf dem
'Datentrger oder korrekten Netzwerkzugriff.

Private Const OFN_OVERWRITEPROMPT = &H2&
'Gibt im Dialog "Speichern" eine Warnmeldung aus, wenn die Datei bereits
'existiert und durch das Speichern berschrieben wird.

Private Const OFN_PATHMUSTEXIST = &H800
'Gibt an, dass der Anwender nur die Namen von existierenden Verzeichnissen
'eingeben kann. Andernfalls wird eine Warnmeldung ausgegeben.

Private Const OFN_READONLY = &H1
'Gibt an, das das Kontrollkstchen "Nur Lesen" angekreuzt ist, wenn der Dialog
'angezeigt wird.

Private Const OFN_SHAREAWARE = &H4000
'Gibt an, dass die Funktion fehlschlgt, wenn ein Netzwerkfehler auftritt.

Private Const OFN_SHAREFALLTHROUGH = 2
Private Const OFN_SHAREWARN = 0
Private Const OFN_SHARENOWARN = 1

Private Const OFN_SHOWHELP = &H10
'Zeigt im Dialogfeld den Hilfe-Schalter an. hwndOwner mu auf ein Fenster zeigen,
'das die Hilfe anzeigen kann. ExplorerDialoge senden die Nachricht CDN_HELP
'an die Rckruffunktion.

Private Const OFS_MAXPATHNAME = 128

'**********************************************************
Rem *** GetOpenFileName ***
Rem Funktion zum Anzeigen des Dialogs "Speichern"

Private Declare Function GetOpenFileName Lib "comdlg32" Alias _
    "GetOpenFileNameA" _
    (lpOpenfilename As OpenFilename) As Long

'**********************************************************
Rem *** CommDlgExtendedError ***
Rem Funktion zum Ermitteln der Fehlernummer

Private Declare Function CommDlgExtendedError Lib "comdlg32" () As Integer

'**********************************************************
Rem *** GetActiveWindow ***
Rem Eine Funktion zum Ermitteln des Fenster-Handles.

Private Declare Function GetActiveWindow Lib "user32" () As Long

'**********************************************************
Rem *** PrepareFilter ***
Rem Eine Funktion zum Aufbereiten des Filters
Rem Beispielfilter: "Word Dokument (.doc)|*.doc|Word Vorlage (.dot)|*.dot|"
Rem Die Funktion ersetzt "|" durch Chr(0) und fgt das abschlieende
Rem Chr(0) ein.

Private Function PrepareFilter(Flt As String) As String
Const Elim = "|"
Dim Temp As String
Dim i As Integer
    'Mit einer Kopie arbeiten
    Temp = Flt
    'Beim ersten Zeichen beginnen
    i = 1
    '"|" gefunden?
    Do While InStr(i, Flt, Elim) <> 0
        'Alles bis zum ersten "|" kopieren und Chr(0) anhngen
        PrepareFilter = PrepareFilter + _
                Mid(Temp, i, InStr(i, Temp, Elim) - i) + vbNullChar
        'Index auf Zeichen nach "|" setzen
        i = InStr(i, Temp, Elim) + Len(Elim)
    Loop
    'Evtl. Rest vom String und abschlieendes Chr(0) anhngen
    PrepareFilter = PrepareFilter + Right(Temp, Len(Temp) - i + 1) + vbNullChar
End Function

'**********************************************************
Rem *** GetOpenName ***
Rem Eine VB/VBA-Funktion als einfach zu nutzender Mantel fr den
Rem Aufruf des Dialogs "ffnen".

Public Function GetOpenName(ByVal Filter As String, ByVal InitialDir As String) As String
Dim OFN As OpenFilename
Dim Temp As String
Dim n As Integer

    'Bestimmen der Optionen fr den Dialog
    With OFN
        .lStructSize = Len(OFN)
        .hWndOwner = GetActiveWindow()
        .lpstrFilter = PrepareFilter(Filter)
        'Speicher reservieren
        .lpstrFile = String(700, vbNullChar)
        .nMaxFile = 700
        .lpstrFileTitle = String(MAX_PATH, vbNullChar)
        .nMaxFileTitle = MAX_PATH
        .lpstrInitialDir = InitialDir
        .lpstrTitle = ""
        .Flags = OFN_EXTENSIONDIFFERENT Or _
            OFN_NOCHANGEDIR Or _
            OFN_OVERWRITEPROMPT Or _
            OFN_HIDEREADONLY Or _
            OFN_ALLOWMULTISELECT Or OFN_EXPLORER 'gehrt zusammen
    End With

    If GetOpenFileName(OFN) Then
        Temp = OFN.lpstrFile
        'Alles nach dem NULL-Zeichen verwerfen
        For n = Len(Temp) To 1 Step -1
            If Mid(Temp, n, 1) <> vbNullChar Then n = n + 1: Exit For
        Next
        If n > 1 Then
            GetOpenName = Left(Temp, n - 1)
        Else
            GetOpenName = ""
        End If
    Else
        GetOpenName = ""
    End If
End Function
